'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

exports.$3dTilesIndex = $3dTilesIndex;

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _Provider = require('./Provider');

var _Provider2 = _interopRequireDefault(_Provider);

var _B3dmLoader = require('../../../Renderer/ThreeExtended/B3dmLoader');

var _B3dmLoader2 = _interopRequireDefault(_B3dmLoader);

var _PntsLoader = require('../../../Renderer/ThreeExtended/PntsLoader');

var _PntsLoader2 = _interopRequireDefault(_PntsLoader);

var _Fetcher = require('./Fetcher');

var _Fetcher2 = _interopRequireDefault(_Fetcher);

var _OBB = require('../../../Renderer/ThreeExtended/OBB');

var _OBB2 = _interopRequireDefault(_OBB);

var _Extent = require('../../Geographic/Extent');

var _Extent2 = _interopRequireDefault(_Extent);

var _MathExtended = require('../../Math/MathExtended');

var _MathExtended2 = _interopRequireDefault(_MathExtended);

var _Capabilities = require('../../System/Capabilities');

var _Capabilities2 = _interopRequireDefault(_Capabilities);

var _PrecisionQualifier = "precision highp float;\nprecision highp int;\n";

var _PrecisionQualifier2 = _interopRequireDefault(_PrecisionQualifier);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function $3dTilesIndex(tileset, urlPrefix) {
    var counter = 0;
    this.index = {};
    var recurse = function recurse_f(node, urlPrefix) {
        this.index[counter] = node;
        node.tileId = counter;
        node.urlPrefix = urlPrefix;
        counter++;
        if (node.children) {
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = (0, _getIterator3.default)(node.children), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var child = _step.value;

                    recurse(child, urlPrefix);
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }
        }
    }.bind(this);
    recurse(tileset.root, urlPrefix);

    this.extendTileset = function extendTileset(tileset, nodeId, urlPrefix) {
        recurse(tileset.root, urlPrefix);
        this.index[nodeId].children = [tileset.root];
    };
}

function $3dTiles_Provider() /* options*/{
    // Constructor

    _Provider2.default.call(this);
    this.b3dmLoader = new _B3dmLoader2.default();
}

$3dTiles_Provider.prototype = (0, _create2.default)(_Provider2.default.prototype);

$3dTiles_Provider.prototype.constructor = $3dTiles_Provider;

$3dTiles_Provider.prototype.removeLayer = function removeLayer() /* idLayer*/{};

$3dTiles_Provider.prototype.preprocessDataLayer = function preprocessDataLayer() /* layer*/{};

function getBox(volume) {
    if (volume.region) {
        var region = volume.region;
        var extent = new _Extent2.default('EPSG:4326', _MathExtended2.default.radToDeg(region[0]), _MathExtended2.default.radToDeg(region[2]), _MathExtended2.default.radToDeg(region[1]), _MathExtended2.default.radToDeg(region[3]));
        var box = _OBB2.default.extentToOBB(extent, region[4], region[5]);
        box.position.copy(box.centerWorld);
        box.updateMatrix();
        box.updateMatrixWorld();
        return { region: box };
    } else if (volume.box) {
        // TODO: only works for axis aligned boxes
        var _box = volume.box;
        // box[0], box[1], box[2] = center of the box
        // box[3], box[4], box[5] = x axis direction and half-length
        // box[6], box[7], box[8] = y axis direction and half-length
        // box[9], box[10], box[11] = z axis direction and half-length
        var center = new THREE.Vector3(_box[0], _box[1], _box[2]);
        var w = center.x - _box[3];
        var e = center.x + _box[3];
        var s = center.y - _box[7];
        var n = center.y + _box[7];
        var b = center.z - _box[11];
        var t = center.z + _box[11];

        return { box: new THREE.Box3(new THREE.Vector3(w, s, b), new THREE.Vector3(e, n, t)) };
    } else if (volume.sphere) {
        var sphere = new THREE.Sphere(new THREE.Vector3(volume.sphere[0], volume.sphere[1], volume.sphere[2]), volume.sphere[3]);
        return { sphere: sphere };
    }
}

var rePosition = new RegExp('gl_Position.*(?![^]*gl_Position)');
var reMain = new RegExp('[^\\w]*main[^\\w]*(void)?[^\\w]*{');
function patchMaterial(material) {
    // Check if the shader does not already use the log depth buffer
    if (material.vertexShader.indexOf('USE_LOGDEPTHBUF') !== -1 || material.vertexShader.indexOf('logdepthbuf_pars_vertex') !== -1) {
        return;
    }

    // Add vertex shader log depth buffer header
    material.vertexShader = '#include <logdepthbuf_pars_vertex>\n#define EPSILON 1e-6\n' + material.vertexShader;
    // Add log depth buffer code snippet after last gl_Position modification
    var re = rePosition.exec(material.vertexShader);
    var idx = re[0].length + re.index;
    material.vertexShader = material.vertexShader.slice(0, idx) + '\n#include <logdepthbuf_vertex>\n' + material.vertexShader.slice(idx);

    // Add fragment shader log depth buffer header
    material.fragmentShader = _PrecisionQualifier2.default + '\n#include <logdepthbuf_pars_fragment>\n' + material.fragmentShader;
    // Add log depth buffer code snippet at the first line of the main function
    re = reMain.exec(material.fragmentShader);
    idx = re[0].length + re.index;
    material.fragmentShader = material.fragmentShader.slice(0, idx) + '\n#include <logdepthbuf_fragment>\n' + material.fragmentShader.slice(idx);

    material.defines = {
        USE_LOGDEPTHBUF: 1,
        USE_LOGDEPTHBUF_EXT: 1
    };

    // eslint-disable-next-line no-console
    console.warn('b3dm shader has been patched to add log depth buffer support');
}

$3dTiles_Provider.prototype.b3dmToMesh = function b3dmToMesh(data, layer) {
    return this.b3dmLoader.parse(data).then(function (result) {
        var init = function f_init(mesh) {
            mesh.frustumCulled = false;
            if (mesh.material) {
                if (layer.overrideMaterials) {
                    mesh.material = new THREE.MeshLambertMaterial(0xffffff);
                } else if (_Capabilities2.default.isLogDepthBufferSupported() && mesh.material.isRawShaderMaterial && !layer.doNotPatchMaterial) {
                    patchMaterial(mesh.material);
                }
            }
        };
        result.scene.applyMatrix(layer.glTFRotation);
        result.scene.traverse(init);
        return result.scene;
    });
};

$3dTiles_Provider.prototype.pntsParse = function pntsParse(data) {
    return new _promise2.default(function (resolve) {
        resolve(_PntsLoader2.default.parse(data));
    });
};

function configureTile(tile, layer, metadata) {
    tile.frustumCulled = false;
    tile.loaded = true;
    tile.layer = layer.id;

    // parse metadata
    tile.transform = metadata.transform ? new THREE.Matrix4().fromArray(metadata.transform) : new THREE.Matrix4();
    tile.applyMatrix(tile.transform);
    tile.geometricError = metadata.geometricError;
    tile.tileId = metadata.tileId;
    tile.additiveRefinement = metadata.refine === 'add';
    tile.boundingVolume = getBox(metadata.boundingVolume);
    tile.viewerRequestVolume = metadata.viewerRequestVolume ? getBox(metadata.viewerRequestVolume) : undefined;
}

var textDecoder = new TextDecoder('utf-8');
$3dTiles_Provider.prototype.executeCommand = function executeCommand(command) {
    var layer = command.layer;
    var metadata = command.metadata;

    var tile = new THREE.Object3D();
    configureTile(tile, layer, metadata);
    var urlSuffix = metadata.content ? metadata.content.url : undefined;
    var setLayer = function setLayer(obj) {
        obj.layers.set(layer.threejsLayer);
    };
    if (urlSuffix) {
        var url = metadata.urlPrefix + urlSuffix;

        var supportedFormats = {
            b3dm: this.b3dmToMesh.bind(this),
            pnts: this.pntsParse.bind(this)
        };

        return _Fetcher2.default.arrayBuffer(url, layer.networkOptions).then(function (result) {
            if (result !== undefined) {
                var func = void 0;
                var magic = textDecoder.decode(new Uint8Array(result, 0, 4));
                if (magic[0] === '{') {
                    result = JSON.parse(textDecoder.decode(new Uint8Array(result)));
                    var newPrefix = url.slice(0, url.lastIndexOf('/') + 1);
                    layer.tileIndex.extendTileset(result, metadata.tileId, newPrefix);
                } else if (magic == 'b3dm') {
                    func = supportedFormats.b3dm;
                } else if (magic == 'pnts') {
                    func = supportedFormats.pnts;
                } else {
                    _promise2.default.reject('Unsupported magic code ' + magic);
                }
                if (func) {
                    // TODO: request should be delayed if there is a viewerRequestVolume
                    return func(result, layer).then(function (content) {
                        tile.add(content);
                        tile.traverse(setLayer);
                        return tile;
                    });
                }
            }

            tile.traverse(setLayer);
            return tile;
        });
    } else {
        return new _promise2.default(function (resolve /* , reject*/) {
            tile.traverse(setLayer);
            resolve(tile);
        });
    }
};

exports.default = $3dTiles_Provider;