'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = loadGpx;

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _Fetcher = require('./Fetcher');

var _Fetcher2 = _interopRequireDefault(_Fetcher);

var _Coordinates = require('../../Geographic/Coordinates');

var _Coordinates2 = _interopRequireDefault(_Coordinates);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _gpxToWayPointsArray(gpxXML) {
    return gpxXML.getElementsByTagName('wpt');
} /**
   * Generated On: 2016-07-07
   * Class: GpxUtils
   * Description: Parse Gpx file to get [lat, lon, alt]
   */

function _gGpxToWTrackPointsArray(gpxXML) {
    return gpxXML.getElementsByTagName('trkpt');
}

function _gpxPtToCartesian(pt) {
    var longitude = Number(pt.attributes.lon.nodeValue);
    var latitude = Number(pt.attributes.lat.nodeValue);
    var elevation = Number(pt.getElementsByTagName('ele')[0].childNodes[0].nodeValue);

    return new _Coordinates2.default('EPSG:4326', longitude, latitude, elevation).as('EPSG:4978').xyz();
}

function _gpxToWayPointsMesh(gpxXML) {
    var wayPts = _gpxToWayPointsArray(gpxXML);

    if (wayPts.length) {
        var positions = new Float32Array(wayPts.length * 3);
        for (var i = 0; i < wayPts.length; i++) {
            var pos = _gpxPtToCartesian(wayPts[i]);
            pos.toArray(positions, 3 * i);
        }
        var points = new THREE.Points();
        points.geometry.addAttribute('position', new THREE.BufferAttribute(positions, 3));
        points.material.color.setRGB(0, 1, 0);
        return points;
    } else {
        return null;
    }
}

function _gpxToWTrackPointsMesh(gpxXML) {
    var trackPts = _gGpxToWTrackPointsArray(gpxXML);

    if (trackPts.length) {
        var positions = new Float32Array(trackPts.length * 3);
        for (var i = 0; i < trackPts.length; i++) {
            var pos = _gpxPtToCartesian(trackPts[i]);
            pos.toArray(positions, 3 * i);
        }

        var line = new THREE.Line();
        line.geometry.addAttribute('position', new THREE.BufferAttribute(positions, 3));
        line.material.color.setRGB(1, 0, 0);
        line.material.linewidth = 100;

        return line;
    } else {
        return null;
    }
}

function _gpxToMesh(gpxXML) {
    if (!gpxXML) {
        return undefined;
    }

    var gpxMesh = new THREE.Object3D();

    // Getting the track points
    var trackPts = _gpxToWTrackPointsMesh(gpxXML);

    if (trackPts) {
        gpxMesh.add(trackPts);
    }

    // Getting the waypoint points
    var wayPts = _gpxToWayPointsMesh(gpxXML);

    if (wayPts) {
        gpxMesh.add(wayPts);
    }

    return gpxMesh;
}

function loadGpx(urlFile, networkOptions) {
    return _Fetcher2.default.xml(urlFile, networkOptions).then(function (gpxXML) {
        return _gpxToMesh(gpxXML);
    });
}