'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

exports.$3dTilesCulling = $3dTilesCulling;
exports.pre3dTilesUpdate = pre3dTilesUpdate;
exports.init3dTilesLayer = init3dTilesLayer;
exports.process3dTilesNode = process3dTilesNode;
exports.$3dTilesSubdivisionControl = $3dTilesSubdivisionControl;

var _three = require('three');

var THREE = _interopRequireWildcard(_three);

var _Fetcher = require('../Core/Scheduler/Providers/Fetcher');

var _Fetcher2 = _interopRequireDefault(_Fetcher);

var _dTiles_Provider = require('../Core/Scheduler/Providers/3dTiles_Provider');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function requestNewTile(view, scheduler, geometryLayer, metadata, parent) {
    var command = {
        /* mandatory */
        view: view,
        requester: parent,
        layer: geometryLayer,
        priority: 10000,
        /* specific params */
        metadata: metadata,
        redraw: false
    };

    return scheduler.execute(command);
}

function subdivideNode(context, layer, node) {
    if (!node.pendingSubdivision && node.children.filter(function (n) {
        return n.layer == layer.id;
    }).length == 0) {
        node.pendingSubdivision = true;

        var childrenTiles = layer.tileIndex.index[node.tileId].children;
        if (childrenTiles === undefined) {
            return;
        }

        var promises = [];
        for (var i = 0; i < childrenTiles.length; i++) {
            promises.push(requestNewTile(context.view, context.scheduler, layer, childrenTiles[i], node).then(function (tile) {
                node.add(tile);
                tile.updateMatrixWorld();
                if (node.additiveRefinement) {
                    context.view.notifyChange(0, true);
                }
            }));
        }

        _promise2.default.all(promises).then(function () {
            node.pendingSubdivision = false;
            context.view.notifyChange(0, true);
        });
    }
}

function $3dTilesCulling(node, camera) {
    // For viewer Request Volume https://github.com/AnalyticalGraphicsInc/3d-tiles-samples/tree/master/tilesets/TilesetWithRequestVolume
    if (node.viewerRequestVolume) {
        var nodeViewer = node.viewerRequestVolume;
        if (nodeViewer.region) {
            // TODO
            return true;
        }
        if (nodeViewer.box) {
            // TODO
            return true;
        }
        if (nodeViewer.sphere) {
            var worldCoordinateCenter = nodeViewer.sphere.center.clone();
            worldCoordinateCenter.applyMatrix4(node.matrixWorld);
            // To check the distance between the center sphere and the camera
            if (!(camera.camera3D.position.distanceTo(worldCoordinateCenter) <= nodeViewer.sphere.radius)) {
                return true;
            }
        }
    }

    // For bounding volume
    if (node.boundingVolume) {
        var boundingVolume = node.boundingVolume;
        if (boundingVolume.region) {
            return !camera.isBox3DVisible(boundingVolume.region.box3D, boundingVolume.region.matrixWorld);
        }
        if (boundingVolume.box) {
            return !camera.isBox3DVisible(boundingVolume.box, node.matrixWorld);
        }
        if (boundingVolume.sphere) {
            return !camera.isSphereVisible(boundingVolume.sphere, node.matrixWorld);
        }
    }
    return false;
}

var preSSE = void 0;
function pre3dTilesUpdate(context) {
    // pre-sse
    var hypotenuse = Math.sqrt(context.camera.width * context.camera.width + context.camera.height * context.camera.height);
    var radAngle = context.camera.camera3D.fov * Math.PI / 180;

    // TODO: not correct -> see new preSSE
    // const HFOV = 2.0 * Math.atan(Math.tan(radAngle * 0.5) / context.camera.ratio);
    var HYFOV = 2.0 * Math.atan(Math.tan(radAngle * 0.5) * hypotenuse / context.camera.width);
    preSSE = hypotenuse * (2.0 * Math.tan(HYFOV * 0.5));
}

// Improved zoom geometry
function computeNodeSSE(camera, node) {
    if (node.boundingVolume.region) {
        var worldCoordinateCenter = node.boundingVolume.region.centerWorld;
        worldCoordinateCenter.applyMatrix4(node.matrixWorld);
        var distance = camera.camera3D.position.distanceTo(worldCoordinateCenter);
        return preSSE * (node.geometricError / distance);
    }
    if (node.boundingVolume.box) {
        // TODO: compute proper distance
        var _worldCoordinateCenter = node.boundingVolume.box.getCenter();
        _worldCoordinateCenter.applyMatrix4(node.matrixWorld);
        var _distance = camera.camera3D.position.distanceTo(_worldCoordinateCenter);
        return preSSE * (node.geometricError / _distance);
    } else if (node.boundingVolume.sphere) {
        var _worldCoordinateCenter2 = node.boundingVolume.sphere.center.clone();
        _worldCoordinateCenter2.applyMatrix4(node.matrixWorld);
        var _distance2 = Math.max(0.0, camera.camera3D.position.distanceTo(_worldCoordinateCenter2) - node.boundingVolume.sphere.radius);
        return preSSE * (node.geometricError / _distance2);
    }
    return Infinity;
}

function init3dTilesLayer(context, layer) {
    if (layer.initialised) {
        return;
    }
    layer.initialised = true;

    _Fetcher2.default.json(layer.url, layer.networkOptions).then(function (tileset) {
        var urlPrefix = layer.url.slice(0, layer.url.lastIndexOf('/') + 1);
        layer.tileIndex = new _dTiles_Provider.$3dTilesIndex(tileset, urlPrefix);
        layer.asset = tileset.asset;
        if (tileset.asset.gltfUpAxis === undefined || tileset.asset.gltfUpAxis === 'Y') {
            layer.glTFRotation = new THREE.Matrix4().makeRotationX(Math.PI / 2);
        } else if (tileset.asset.gltfUpAxis === 'X') {
            layer.glTFRotation = new THREE.Matrix4().makeRotationZ(-Math.PI / 2);
        } else if (tileset.asset.gltfUpAxis === 'Z') {
            layer.glTFRotation = new THREE.Matrix4();
        }
        requestNewTile(context.view, context.scheduler, layer, tileset.root, undefined).then(function (tile) {
            context.view.scene.add(tile);
            tile.updateMatrixWorld();
            layer.root = tile;
        });
    });
}

function process3dTilesNode(cullingTest, subdivisionTest) {
    return function _process3dTilesNodes(context, layer, node) {
        // early exit if parent' subdivision is in progress
        if (node.parent.pendingSubdivision && !node.parent.additiveRefinement) {
            node.visible = false;
            // TODO: node.setDisplayed(false)?
            if (node.material) {
                node.material.visible = false;
            }
            return undefined;
        }

        // do proper culling
        var isVisible = cullingTest ? !cullingTest(node, context.camera) : true;
        node.visible = isVisible;

        var returnValue = void 0;

        if (isVisible) {
            if (node.pendingSubdivision || subdivisionTest(context, layer, node)) {
                subdivideNode(context, layer, node);
                // display iff children aren't ready
                if (node.material) {
                    node.material.visible = node.pendingSubdivision || node.additiveRefinement;
                }
                returnValue = node.children.filter(function (n) {
                    return n.layer == layer.id;
                });
            } else if (node.material) {
                node.material.visible = true;
            }

            if (node.material === undefined || node.material.visible) {
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = (0, _getIterator3.default)(node.children.filter(function (n) {
                        return n.layer == layer.id;
                    })), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var n = _step.value;

                        n.visible = false;
                        if (n.material) {
                            n.material.visible = false;
                        }
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator.return) {
                            _iterator.return();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }
            }

            return returnValue;
        }

        if (node.material) {
            node.material.visible = false;
        }

        // TODO: cleanup tree
        return undefined;
    };
}

function $3dTilesSubdivisionControl(context, layer, node) {
    var sse = computeNodeSSE(context.camera, node);

    return sse > 1.0;
}